import os
import pytest

from punch import config as pc


@pytest.fixture
def config_file_content():
    return """
__config_version__ = 1

# http://semver.org/
GLOBALS = {
    'serializer': '{{major}}.{{minor}}.{{patch}}'
}

FILES = [
    'pkg/__init__.py',
    {
        'path': 'version.txt',
        'serializer': '{{major}}.{{minor}}'
    }
]

VERSION = [
    {
        'name': 'major',
        'type': 'integer'
    },
    {
        'name': 'minor',
        'type': 'integer'
    },
    {
        'name': 'patch',
        'type': 'integer'
    }
]
"""


@pytest.fixture
def config_file_content_with_vcs(config_file_content):
    return config_file_content + """

VCS = {
    'name': 'git',
    'commit_message': "Version updated to {{ new_version }}",
    'options': {
        'make_release_branch': False,
        'annotate_tags': False,
        'annotation_message': '',
    }
}
"""


@pytest.fixture
def config_file_content_with_wrong_vcs(config_file_content):
    return config_file_content + """

VCS = {
    'commit_message': "Version updated to {{ new_version }}",
    'options': {
        'make_release_branch': False,
        'annotate_tags': False,
        'annotation_message': '',
    }
}
"""


@pytest.fixture
def empty_file_content():
    return """
"""


@pytest.fixture
def version_file_content():
    return """
major = 1
minor = 5
patch = 0
"""


@pytest.fixture
def illegal_config_file_content():
    return """
__config_version__ = 2
"""


@pytest.fixture
def config_file_name():
    return 'punch_config.py'


@pytest.fixture
def version_file_name():
    return 'punch_version.py'


def clean_previous_imports():
    import sys

    for i in ['punch_config', 'punch_version']:
        if i in sys.modules:
            sys.modules.pop(i)


def write_file(dir, content, config_file_name):
    with open(os.path.join(dir, config_file_name), 'w') as f:
        f.write(content)


def test_read_empty_config_file(temp_empty_dir, empty_file_content, config_file_name,
                                version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, empty_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    with pytest.raises(ValueError) as exc:
        pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert str(exc.value) == "Given config file is invalid: missing '__config_version__' variable"


def test_read_illegal_config_file(temp_empty_dir, illegal_config_file_content, config_file_name,
                                  version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, illegal_config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    with pytest.raises(pc.ConfigurationVersionError) as exc:
        pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert str(exc.value) == "Unsupported configuration file version 2"


def test_read_plain_variables(temp_empty_dir, config_file_content, config_file_name,
                              version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert cf.__config_version__ == 1


def test_read_global_variables(temp_empty_dir, config_file_content, config_file_name,
                               version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    expected_dict = {
        'serializer': '{{major}}.{{minor}}.{{patch}}'
    }

    assert cf.globals == expected_dict


def test_read_files(temp_empty_dir, config_file_content, config_file_name,
                    version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert len(cf.files) == 2
    assert [fc.path for fc in cf.files] == ['pkg/__init__.py', 'version.txt']


def test_read_version(temp_empty_dir, config_file_content, config_file_name,
                      version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    expected_value = [
        {
            'name': 'major',
            'type': 'integer'
        },
        {
            'name': 'minor',
            'type': 'integer'
        },
        {
            'name': 'patch',
            'type': 'integer'
        }
    ]

    assert cf.version == expected_value


def test_read_config_missing_vcs(temp_empty_dir, config_file_content, config_file_name,
                                 version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert cf.vcs is None


def test_read_vcs(temp_empty_dir, config_file_content_with_vcs, config_file_name,
                  version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content_with_vcs, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    cf = pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    expected_dict = {
        'name': 'git',
        'commit_message': "Version updated to {{ new_version }}",
        'options': {
            'make_release_branch': False,
            'annotate_tags': False,
            'annotation_message': '',
        }
    }

    assert cf.vcs == expected_dict


def test_read_vcs_missing_name(temp_empty_dir, config_file_content_with_wrong_vcs, config_file_name,
                               version_file_content, version_file_name):
    clean_previous_imports()

    write_file(temp_empty_dir, config_file_content_with_wrong_vcs, config_file_name)
    write_file(temp_empty_dir, version_file_content, version_file_name)

    with pytest.raises(ValueError) as exc:
        pc.PunchConfig(os.path.join(temp_empty_dir, config_file_name))

    assert str(exc.value) == "Missing key 'name' in VCS configuration"



import mock

from punch.vcs_use_cases import tag


def test_pre_tag():
    repo = mock.Mock()
    use_case = tag.VCSTagUseCase(repo)

    use_case.tag("just_a_tag")

    assert repo.tag.called_with("just_a_tag")



# -*- coding: utf-8 -*-

import pytest

from punch import version_part as vpart


def test_integer_version_part_init_with_integer():
    vp = vpart.IntegerVersionPart('major', 4)
    assert vp.value == 4


def test_integer_version_part_init_with_string():
    vp = vpart.IntegerVersionPart('major', '4')
    assert vp.value == 4


def test_integer_version_part_init_with_none():
    vp = vpart.IntegerVersionPart('major', None)
    assert vp.value == 0


def test_integer_version_part_init_without_value():
    vp = vpart.IntegerVersionPart('major')
    assert vp.value == 0


def test_integer_version_part_init_with_start_value():
    vp = vpart.IntegerVersionPart('major', start_value=1)
    assert vp.value == 1
    vp.inc()
    vp.reset()
    assert vp.value == 1


def test_integer_version_part_increases():
    vp = vpart.IntegerVersionPart('major', 4)
    vp.inc()
    assert vp.value == 5


def test_integer_version_part_set():
    vp = vpart.IntegerVersionPart('major', 4)
    vp.set(9)
    assert vp.value == 9


def test_integer_version_part_reset():
    vp = vpart.IntegerVersionPart('major', 4)
    vp.reset()
    assert vp.value == 0


def test_integer_version_part_copy():
    vp = vpart.IntegerVersionPart('major', 4)
    nvp = vp.copy()
    vp.inc()

    assert nvp.value == 4


def test_integer_version_part_with_start_value_copy():
    vp = vpart.IntegerVersionPart('major', 4, start_value=1)
    nvp = vp.copy()

    assert nvp.start_value == 1


def test_valuelist_version_part_init_with_allowed_value():
    vp = vpart.ValueListVersionPart('major', 0, [0, 2, 4, 6, 8])
    assert vp.value == 0


def test_valuelist_version_part_init_with_not_allowed_value():
    with pytest.raises(ValueError):
        vpart.ValueListVersionPart('major', 1, [0, 2, 4, 6, 8])


def test_valuelist_version_part_init_with_none():
    vp = vpart.ValueListVersionPart('major', None, [0, 2, 4, 6, 8])
    assert vp.value == 0


def test_valuelist_version_part_increase():
    vp = vpart.ValueListVersionPart('major', 0, [0, 2, 4, 6, 8])
    vp.inc()
    assert vp.value == 2


def test_valuelist_version_part_set():
    vp = vpart.ValueListVersionPart('major', 0, [0, 2, 4, 6, 8])
    vp.set(8)
    assert vp.value == 8


def test_valuelist_version_part_increase_from_last():
    vp = vpart.ValueListVersionPart('major', 8, [0, 2, 4, 6, 8])
    vp.inc()
    assert vp.value == 0


def test_valuelist_version_part_increase_with_non_numerical_values():
    vp = vpart.ValueListVersionPart('major', 0, [0, 'alpha', 'beta', 'rc1', 'rc2', 1])
    vp.inc()
    assert vp.value == 'alpha'


def test_valuelist_version_part_set_with_non_numerical_values():
    vp = vpart.ValueListVersionPart('major', 0, [0, 'alpha', 'beta', 'rc1', 'rc2', 1])
    vp.set('rc1')
    assert vp.value == 'rc1'


def test_valuelist_version_part_reset():
    vp = vpart.ValueListVersionPart('major', 4, [0, 2, 4, 6, 8])
    vp.reset()
    assert vp.value == 0


def test_valuelist_version_part_copy():
    vp = vpart.ValueListVersionPart('major', 4, [0, 2, 4, 6, 8])
    nvp = vp.copy()
    vp.inc()
    vp.values.append(9)

    assert nvp.value == 4
    assert nvp.values == [0, 2, 4, 6, 8]


def test_get_integer_version_part_from_full_dict():
    input_dict = {
        'name': 'major',
        'value': 1,
        'type': 'integer'
    }

    vp = vpart.VersionPart.from_dict(input_dict)

    assert vp.name == 'major'
    assert vp.value == 1
    assert isinstance(vp, vpart.IntegerVersionPart)


def test_get_integer_version_part_from_partial_dict():
    input_dict = {
        'name': 'major',
        'value': 1,
    }

    vp = vpart.VersionPart.from_dict(input_dict)

    assert vp.name == 'major'
    assert vp.value == 1
    assert isinstance(vp, vpart.IntegerVersionPart)


def test_get_value_list_version_part_from_full_dict():
    input_dict = {
        'name': 'major',
        'value': 'alpha',
        'type': 'value_list',
        'allowed_values': ['alpha', 'beta', 'stable']
    }

    vp = vpart.VersionPart.from_dict(input_dict)

    assert vp.name == 'major'
    assert vp.value == 'alpha'
    assert isinstance(vp, vpart.ValueListVersionPart)
    assert vp.values == ['alpha', 'beta', 'stable']



import os
import subprocess
import textwrap
import tempfile
import shutil
import re

import pytest

# This part of the test suite is in development, the aim is to find a way to easily test
# a script included in a Python package through pytest.

class TestEnvironment(object):
    def __init__(self, path):
        self.finalize = True
        self.path = path

    def ensure_file_is_present(self, filename, content=""):
        with open(os.path.join(self.path, filename), "w") as f:
            f.write(textwrap.dedent(content))

    def get_file_content(self, filename):
        with open(os.path.join(self.path, filename), "r") as f:
            return f.read()

    def call(self, cmdline, **kwds):
        return subprocess.check_call(cmdline, cwd=self.path, **kwds)

    def output(self, cmdline, **kwds):
        output = subprocess.check_output(cmdline, cwd=self.path, **kwds)
        return output.decode('utf8')

    def compare_output(self, expected_output, cmdline, **kwds):
        output = self.output(cmdline, **kwds)

        clean_expected_output = textwrap.dedent(expected_output)

        for line in clean_expected_output.split(os.linesep):
            reline = re.compile(line)
            l = reline.findall(output)
            assert len(l) > 0


@pytest.fixture
def test_environment(request):
    tempdir = tempfile.mkdtemp()

    te = TestEnvironment(tempdir)

    def fin():
        if te.finalize:
            shutil.rmtree(tempdir, ignore_errors=True)

    request.addfinalizer(fin)
    return te





# -*- coding: utf-8 -*-

import collections
from punch import version_part as vpart
from punch.helpers import import_file


class Version():
    def __init__(self):
        self.parts = collections.OrderedDict()

    @property
    def keys(self):
        return list(self.parts.keys())

    @property
    def values(self):
        return list(self.parts.values())

    def add_part(self, part):
        self.keys.append(part.name)
        self.parts[part.name] = part

    def create_part(self, name, value, cls=vpart.IntegerVersionPart, *args, **kwds):
        self.keys.append(name)
        self.parts[name] = cls(name, value, *args, **kwds)

    def add_part_from_dict(self, dic):
        vp = vpart.VersionPart.from_dict(dic)
        self.keys.append(vp.name)
        self.parts[vp.name] = vp

    def get_part(self, name):
        return self.parts[name]

    def _reset_following_parts(self, name):
        idx = self.keys.index(name)
        reset_keys = self.keys[idx + 1:]
        for key in reset_keys:
            self.parts[key].reset()

    def inc(self, name):
        self.parts[name].inc()
        self._reset_following_parts(name)

    def set(self, adict):
        for key, value in adict.items():
            self.parts[key].set(value)

    def set_and_reset(self, name, value):
        self.parts[name].set(value)
        self._reset_following_parts(name)

    def copy(self):
        new = Version()
        for value in self.parts.values():
            new.add_part(value.copy())

        return new

    def as_dict(self):
        return dict((key, part.value) for key, part in self.parts.items())

    def as_list(self):
        return list((key, part.value) for key, part in self.parts.items())

    def to_file(self, version_filepath):
        with open(version_filepath, 'w') as f:
            for key, part in self.parts.items():
                f.write("{0} = {1}\n".format(key, part.value))

    @classmethod
    def from_file(cls, version_filepath, version_description):
        version_module = import_file(version_filepath)
        version = Version()

        for version_part in version_description:
            if isinstance(version_part, collections.Mapping):
                version_part_name = version_part['name']
                version_part['value'] = cls._get_version_part(version_module, version_part, version_part_name)
                version.add_part_from_dict(version_part)
            else:
                version_part_name = version_part
                version_part_value = cls._get_version_part(version_module, version_part, version_part_name)
                version.create_part(version_part_name, version_part_value)

        return version

    @classmethod
    def _get_version_part(cls, version_module, version_part, version_part_name):
        try:
            return getattr(version_module, version_part_name)
        except AttributeError:
            raise ValueError("Given version file is invalid: missing '{}' variable".format(version_part_name))



import pytest

pytestmark = pytest.mark.slow

version_file_content = """
major = 1
minor = 0
patch = 0
"""

config_file_content = """
__config_version__ = 1

GLOBALS = {
    'serializer': '{{major}}.{{minor}}.{{patch}}',
}

FILES = ["README.md"]

VERSION = ['major', 'minor', 'patch']
"""

def test_update_major(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "major"])

    assert test_environment.get_file_content("README.md") == "Version 2.0.0"


def test_update_minor(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "minor"])

    assert test_environment.get_file_content("README.md") == "Version 1.1.0"


def test_update_patch(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "patch"])

    assert test_environment.get_file_content("README.md") == "Version 1.0.1"



import sys


def import_file(filepath):
    if sys.version_info < (3, 0):
        import imp

        module = imp.load_source("punch_config", filepath)

    elif sys.version_info < (3, 5):
        from importlib.machinery import SourceFileLoader

        try:
            module = SourceFileLoader("punch_config", filepath).load_module()
        except FileNotFoundError:
            raise ValueError("The module file {} cannot be found.".format(filepath))
        except ImportError:
            raise ValueError(
                "The module file {} cannot imported due to an error.".format(filepath))

    else:
        import importlib.util

        spec = importlib.util.spec_from_file_location("punch_config", filepath)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

    return module



import subprocess

import pytest


def test_punch_version_flag(test_environment):
    # output = test_environment_fixture.output(["punch", "--version"])

    # Punch version 1.0.1
    # Copyright (C) 2016 Leonardo Giordani

    expected_output = """
    Copyright \(C\) \d{4} Leonardo Giordani
    This is free software, see the LICENSE file.
    Source: https://github.com/lgiordani/punch
    Documentation: http://punch.readthedocs.io/en/latest/
    """

    test_environment.compare_output(expected_output, ["punch", "--version"])


def test_punch_increase_version_part(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py",
                                            "major = 1\nminor = 0\npatch = 0\n")

    config_file_content = """
    __config_version__ = 1

    GLOBALS = {
        'serializer': '{{major}}.{{minor}}.{{patch}}',
    }

    FILES = ["README.md"]

    VERSION = ['major', 'minor', 'patch']
    """

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "major"])

    assert test_environment.get_file_content("README.md") == "Version 2.0.0"


def test_punch_set_version_part(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py",
                                            "major = 1\nminor = 0\npatch = 0\n")

    config_file_content = """
    __config_version__ = 1

    GLOBALS = {
        'serializer': '{{major}}.{{minor}}.{{patch}}',
    }

    FILES = ["README.md"]

    VERSION = ['major', 'minor', 'patch']
    """

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--set-part", "minor=4"])

    assert test_environment.get_file_content("README.md") == "Version 1.4.0"


def test_punch_set_multiple_version_parts(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0")

    test_environment.ensure_file_is_present("punch_version.py",
                                            "major = 1\nminor = 0\npatch = 0\n")

    config_file_content = """
    __config_version__ = 1

    GLOBALS = {
        'serializer': '{{major}}.{{minor}}.{{patch}}',
    }

    FILES = ["README.md"]

    VERSION = ['major', 'minor', 'patch']
    """

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--set-part", "minor=4,patch=23"])

    assert test_environment.get_file_content("README.md") == "Version 1.4.23"


def test_punch_set_and_reset_single_part(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.2.3")

    test_environment.ensure_file_is_present("punch_version.py",
                                            "major = 1\nminor = 2\npatch = 3\n")

    config_file_content = """
    __config_version__ = 1

    GLOBALS = {
        'serializer': '{{major}}.{{minor}}.{{patch}}',
    }

    FILES = ["README.md"]

    VERSION = ['major', 'minor', 'patch']
    """

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--set-part", "major=9", "--reset-on-set"])

    assert test_environment.get_file_content("README.md") == "Version 9.0.0"


def test_punch_set_and_reset_multiple_parts_fails(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.2.3")

    test_environment.ensure_file_is_present("punch_version.py",
                                            "major = 1\nminor = 2\npatch = 3\n")

    config_file_content = """
    __config_version__ = 1

    GLOBALS = {
        'serializer': '{{major}}.{{minor}}.{{patch}}',
    }

    FILES = ["README.md"]

    VERSION = ['major', 'minor', 'patch']
    """

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    with pytest.raises(subprocess.CalledProcessError):
        test_environment.output(["punch", "--set-part", "major=9,minor=8", "--reset-on-set"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    # TODO: put package requirements here
]

test_requirements = [
    'pytest',
    'tox'
]

setup(
    name='punch.py',
    version='1.1.0',
    description="Update your version while having a drink",
    long_description=readme + '\n\n' + history,
    author="Leonardo Giordani",
    author_email='giordani.leonardo@gmail.com',
    url='https://github.com/lgiordani/punch',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords=['version', 'management'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    scripts=[
        'scripts/punch',
    ]
)



import subprocess

import os
import six
from punch.vcs_repositories import git_repo as gr
from punch.vcs_repositories.exceptions import RepositoryStatusError, RepositorySystemError


class GitFlowRepo(gr.GitRepo):
    def __init__(self, working_path, config_obj):
        if six.PY2:
            super(GitFlowRepo, self).__init__(working_path, config_obj)
        else:
            super().__init__(working_path, config_obj)

    def _set_command(self):
        self.commands = ['git', 'flow']
        self.command = 'git'

    def _check_system(self):
        # git flow -h returns 1 so the call fails

        p = subprocess.Popen(self.commands, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        stdout, stderr = p.communicate()

        if "git flow <subcommand>" not in stdout.decode('utf8'):
            raise RepositorySystemError("Cannot run {}".format(self.commands))

        if not os.path.exists(os.path.join(self.working_path, '.git')):
            raise RepositorySystemError("The current directory {} is not a Git repository".format(self.working_path))

    def pre_start_release(self):
        output = self._run([self.command, "status"])
        if "Changes to be committed:" in output:
            raise RepositoryStatusError("Cannot checkout master while repository contains uncommitted changes")

        self._run([self.command, "checkout", "develop"])

        branch = self.get_current_branch()

        if branch != "develop":
            raise RepositoryStatusError("Current branch shall be master but is {}".format(branch))

    def start_release(self):
        self._run(self.commands + ["release", "start", self.config_obj.options['new_version']])

    def finish_release(self):
        branch = self.get_current_branch()

        self._run([self.command, "add", "."])

        output = self._run([self.command, "status"])
        if "nothing to commit, working directory clean" in output:
            self._run([self.command, "checkout", "develop"])
            self._run([self.command, "branch", "-d", branch])
            return

        message = ["-m", self.config_obj.commit_message]

        command_line = [self.command, "commit"]
        command_line.extend(message)

        self._run(command_line)

        self._run(self.commands + ["release", "finish", "-m", branch, self.config_obj.options['new_version']])

    def post_finish_release(self):
        pass



import pytest

pytestmark = pytest.mark.slow

version_file_content = """
major = 1
minor = 0
patch = 0
prerelease = 'alpha'
"""

config_file_content = """
__config_version__ = 1

GLOBALS = {
    'serializer': "{{ major }}.{{ minor }}.{{ patch }}{{ '-{}'.format(prerelease) if prerelease }}"
}

FILES = ["README.md"]

VERSION = [
            'major',
            'minor',
            'patch',
            {
                'name': 'prerelease',
                'type': 'value_list',
                'allowed_values': ['', 'alpha', 'beta']
            }
          ]
"""


def test_update_major(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0-alpha")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "major"])

    assert test_environment.get_file_content("README.md") == "Version 2.0.0"


def test_update_minor(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0-alpha")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "minor"])

    assert test_environment.get_file_content("README.md") == "Version 1.1.0"


def test_update_patch(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0-alpha")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "patch"])

    assert test_environment.get_file_content("README.md") == "Version 1.0.1"


def test_update_prerelease(test_environment):
    test_environment.ensure_file_is_present("README.md", "Version 1.0.0-alpha")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "prerelease"])

    assert test_environment.get_file_content("README.md") == "Version 1.0.0-beta"

def test_update_after_last_prerelease(test_environment):
    version_file_content = """
    major = 1
    minor = 0
    patch = 0
    prerelease = 'beta'
    """

    test_environment.ensure_file_is_present("README.md", "Version 1.0.0-beta")

    test_environment.ensure_file_is_present("punch_version.py", version_file_content)

    test_environment.ensure_file_is_present("punch_config.py", config_file_content)

    test_environment.call(["punch", "--part", "prerelease"])

    assert test_environment.get_file_content("README.md") == "Version 1.0.0"



import jinja2


class FileConfiguration(object):
    def __init__(self, filepath, local_variables, global_variables=None):
        self.config = {}
        if global_variables:
            self.config.update(global_variables)

        new_local_variables = {}
        env = jinja2.Environment(undefined=jinja2.DebugUndefined)
        for key, value in local_variables.items():
            template = env.from_string(value)
            new_local_variables[key] = template.render(GLOBALS=global_variables)

        self.config.update(new_local_variables)
        self.path = filepath

    @classmethod
    def from_dict(cls, dic, global_variables):
        return cls(dic['path'], dic, global_variables)
